UNIT KCtrls;
{+H
---------------------------------------------------------------------------
  File        - KCTRLS.PAS
  Purpose     - Kustom components by Keith Brown.
  Author      - Keith S. Brown. (Houston, TX, USA)
                E-mail: keith_brown@juno.com
                http://www.ghg.net/chairouchu/
                2437 Bay Area Blvd #20, Houston TX / USA
  Copyright    1998 Keith S. Brown
  Restrictions- This component is FREEWARE!
                No warranty is given by the author, expressed or implied.
  Revised     - 1998.0506 (KSB) Wrote initial version.
              - 1998.0602 (KSB) Renamed types for bevelWidth, borderWidth &
                borderStyle to prevent conflicts. Revised Paint method.
---------------------------------------------------------------------------}
INTERFACE
USES
  Messages, Classes,  Controls, StdCtrls, Graphics;



CONST
  xBoxColor   = 1;
  xTxtColor   = 2;

TYPE
  TKCtrlName  = (_TKGroupBox);
  TGroupBevel = (bvNone,bvLowered,bvRaised);
  TGroupBevelWidth = 1..MaxInt;
  TGroupBorderWidth= 0..MaxInt;
  TGroupBorderStyle= (bsNone,bsSingle,bsBeveled);


  TKustomGroupBox  = CLASS(TCustomGroupBox)
  PRIVATE
    FfParentColor  : BOOLEAN;
    FiBoxColor     : TColor;
    FiTxtColor     : TColor;
    FAbout         : TObject;
    FAlignment     : TAlignment;
    FBevelInner    : TGroupBevel;
    FBevelOuter    : TGroupBevel;
    FBevelWidth    : TGroupBevelWidth;
    FBorderWidth   : TGroupBorderWidth;
    FBorderStyle   : TGroupBorderStyle;

    PROCEDURE   SetAlignment(aValue: TAlignment);
    PROCEDURE   SetBevelInner(aValue: TGroupBevel);
    PROCEDURE   SetBevelOuter(aValue: TGroupBevel);
    PROCEDURE   SetBevelWidth(aValue: TGroupBevelWidth);
    PROCEDURE   SetBorderWidth(aValue: TGroupBorderWidth);
    PROCEDURE   SetBorderStyle(aValue: TGroupBorderStyle);
    PROCEDURE   SetColor(aIndex: INTEGER;aValue: TColor);
    PROCEDURE   SetkParentColor(aValue:BOOLEAN);
    PROCEDURE   CMParentColorChanged(VAR aMessage: TMessage); MESSAGE CM_PARENTCOLORCHANGED;
  PROTECTED
    PROCEDURE   Paint; OVERRIDE;
    PROPERTY    Alignment: TAlignment READ FAlignment WRITE SetAlignment DEFAULT taLeftJustify;
    PROPERTY    BevelInner: TGroupBevel READ FBevelInner WRITE SetBevelInner DEFAULT bvNone;
    PROPERTY    BevelOuter: TGroupBevel READ FBevelOuter WRITE SetBevelOuter DEFAULT bvRaised;
    PROPERTY    BevelWidth: TGroupBevelWidth READ FBevelWidth WRITE SetBevelWidth DEFAULT 1;
    PROPERTY    BorderWidth: TGroupBorderWidth READ FBorderWidth WRITE SetBorderWidth DEFAULT 0;
    PROPERTY    BorderStyle: TGroupBorderStyle READ FBorderStyle WRITE SetBorderStyle DEFAULT bsSingle;
    PROPERTY    BoxColor: TColor INDEX xBoxColor READ FiBoxColor WRITE SetColor DEFAULT clBtnFace;
    PROPERTY    ParentColor:BOOLEAN READ FfParentColor WRITE SetkParentColor DEFAULT False;
    PROPERTY    TextColor: TColor INDEX xTxtColor READ FiTxtColor WRITE SetColor DEFAULT clBtnFace;
  PUBLIC
    CONSTRUCTOR Create(aOwner: TComponent); OVERRIDE;
  PUBLISHED
    PROPERTY    About : TObject READ FAbout WRITE FAbout;
  END {CLASS};


  TKGroupBox  = CLASS(TKustomGroupBox)
    PROPERTY    Align;
    PROPERTY    Alignment;
    PROPERTY    BevelInner;
    PROPERTY    BevelOuter;
    PROPERTY    BevelWidth;
    PROPERTY    BorderStyle;
    PROPERTY    BorderWidth;
    PROPERTY    BoxColor;
    PROPERTY    Caption;
    PROPERTY    Ctl3D;
    PROPERTY    DragCursor;
    PROPERTY    DragMode;
    PROPERTY    Enabled;
    PROPERTY    Font;
    PROPERTY    ParentColor;
    PROPERTY    ParentCtl3D;
    PROPERTY    ParentFont;
    PROPERTY    ParentShowHint;
    PROPERTY    PopupMenu;
    PROPERTY    ShowHint;
    PROPERTY    TabOrder;
    PROPERTY    TabStop;
    PROPERTY    TextColor;
    PROPERTY    Visible;
    PROPERTY    OnClick;
    PROPERTY    OnDblClick;
    PROPERTY    OnDragDrop;
    PROPERTY    OnDragOver;
    PROPERTY    OnEndDrag;
    PROPERTY    OnEnter;
    PROPERTY    OnExit;
    PROPERTY    OnMouseDown;
    PROPERTY    OnMouseMove;
    PROPERTY    OnMouseUp;
    PROPERTY    OnStartDrag;
  END {CLASS};


{}FUNCTION    GetCopyright(aCtrlName:TKCtrlName):STRING;
{}FUNCTION    GetVersion(aCtrlName:TKCtrlName):STRING;
              {=================================================}
IMPLEMENTATION
USES
  Windows,  ExtCtrls;




{}FUNCTION    GetCopyright(aCtrlName:TKCtrlName):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Copyright string for About property.
---------------------------------------------------------------------------}
  BEGIN
    result := 'Freeware (Copyright  1998) By Keith Brown'#13;
    CASE aCtrlName OF
      _TKGroupBox : result := result +'Inner/Outer Beveled Group Box'#13;
      ELSE          result := result +'** CtrlName error **';
    END {CASE};

    result := result +'E-Mail: keith_brown@juno.com';
{}END {GetCopyright};




{}FUNCTION    GetVersion(aCtrlName:TKCtrlName):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Version string for About property.
---------------------------------------------------------------------------}
  BEGIN
    CASE aCtrlName OF
      _TKGroupBox : result := 'TKustomGroupBox version 0.0.0';
      ELSE          result := '** CtrlName error **';
    END {CASE};
{}END {GetVersion};




TYPE
  TKontrol    = CLASS(TControl)
    PROPERTY    Color;
  END {CLASS};


{}CONSTRUCTOR TKustomGroupBox.Create(aOwner:TComponent);
{+M
---------------------------------------------------------------------------
  Remarks     - Initialize fields and properties.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create(aOwner);
    ControlStyle := [csAcceptsControls,csCaptureMouse,csClickEvents,csSetCaption, {csOpaque,}csDoubleClicks,csReplicatable];
    Width := 185;
    Height := 105;
    BevelOuter := bvRaised;
    BevelWidth := 1;
    FBorderStyle := bsSingle;
    FfParentColor := FALSE;
    BoxColor := clBtnFace;
    TextColor := clBtnFace;
{}END {Create};




{}PROCEDURE   TKustomGroupBox.Paint;
{+M
---------------------------------------------------------------------------
  Remarks     - The canvas.brush.style should not be set to bsClear. Doing
                so will prevent the box from assuming the value of FiBoxColor.
  Revised     - 1998.0602 (KSB) Removed the 2 pixel bias from all sides of
                the rectangle (except the top).
---------------------------------------------------------------------------}
  VAR
    liBottomColor  : TColor;
    liTopColor: TColor;


{}{}PROCEDURE AdjustColors(Bevel:TGroupBevel);
    BEGIN
      IF Bevel = bvLowered THEN BEGIN
        liTopColor := clBtnShadow;
        liBottomColor := clBtnHighlight;
      END ELSE BEGIN
        liTopColor := clBtnHighlight;
        liBottomColor := clBtnShadow;
      END {IF};
{}{}END {AdjustColors};


  VAR
    laRect    : TRect;
    liFontHt  : INTEGER;
    liX       : INTEGER;
    liY       : INTEGER;
  BEGIN
    WITH Canvas DO BEGIN
      IF Parent <> NIL THEN BEGIN
        Brush.Color := TKontrol(Parent).color;
        laRect := Rect(0,0,Width,Height);
        FillRect(laRect);
      END {IF};

      Font := Self.Font;
      liFontHt := TextHeight('W');
      Brush.Color := FiBoxColor;

      CASE FBorderStyle OF
        bsNone: BEGIN
          IF Text = '' THEN BEGIN
            liY := 0;
          END ELSE BEGIN
            liY := liFontHt;
          END {IF};

          laRect := Rect(0,liY,Width,Height);
          FillRect(laRect);
        END {IF};


        bsSingle   : BEGIN
{         laRect := Rect(2,(liFontHt DIV 2)-1,Width-2,Height-2);}
          laRect := Rect(0,(liFontHt DIV 2)-1,Width,Height);
          FillRect(laRect);                 { TCanvas }
          IF Ctl3D THEN BEGIN
            Inc(laRect.Left);
            Inc(laRect.Top);
            Brush.Color := clBtnHighlight;
            FrameRect(laRect);              { TCanvas }
            OffsetRect(laRect,-1,-1);       { -- WinAPI }
            Brush.Color := clBtnShadow;
          END ELSE BEGIN
            Brush.Color := clWindowFrame;
          END {IF};

          FrameRect(laRect);
        END {IF};


        bsBeveled  : BEGIN
{         laRect := Rect(2,2,Width-2,Height-2);}
          laRect := Rect(0,2,Width,Height);
          FillRect(laRect);
          IF BevelOuter <> bvNone THEN BEGIN
            AdjustColors(BevelOuter);
            Frame3D(Canvas,laRect,liTopColor,liBottomColor,BevelWidth);
          END {IF};

          Frame3D(Canvas,laRect,BoxColor,BoxColor,BorderWidth);
          IF BevelInner <> bvNone THEN BEGIN
            AdjustColors(BevelInner);
            Frame3D(Canvas,laRect,liTopColor,liBottomColor,BevelWidth);
          END {IF};
        END;
      END {CASE};

      IF Text <> '' THEN BEGIN
        CASE FAlignment OF
          taLeftJustify : liX := 8;
          taRightJustify: liX := Width-TextWidth(Text)-8;
          taCenter      : liX := (Width-TextWidth(Text)) DIV 2;
          ELSE            liX := 0;
        END {CASE};

        laRect := Rect(liX,0,0,liFontHt);
      {DrawText is a WinAPI call. The DT_CALCRECT flag forces DrawText to modify the
       width & height of the bounding rectangle, but does not draw the text, thus the
       second call to DrawText. DT_LEFT forces left-justification in the rectangle
       and DT_SINGLELINE forces the text to lie in a single line (<CR> & <LF> ignored)}

        DrawText(Handle,PChar(Text),Length(Text),laRect,DT_LEFT OR DT_SINGLELINE OR DT_CALCRECT);
        Brush.Color := TextColor;
        DrawText(Handle,PChar(Text),Length(Text),laRect,DT_LEFT OR DT_SINGLELINE);
      END {IF};
    END {WITH};
{}END {Paint};




{}PROCEDURE   TKustomGroupBox.SetAlignment(aValue:TAlignment);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    FAlignment := aValue;
    Invalidate;
{}END {SetAlignment};




{}PROCEDURE   TKustomGroupBox.SetBevelInner(aValue:TGroupBevel);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    FBevelInner := aValue;
    Realign;
    Invalidate;
{}END {SetBevelInner};




{}PROCEDURE   TKustomGroupBox.SetBevelOuter(aValue:TGroupBevel);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    FBevelOuter := aValue;
    Realign;
    Invalidate;
{}END {SetBevelOuter};




{}PROCEDURE   TKustomGroupBox.SetBevelWidth(aValue:TGroupBevelWidth);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    FBevelWidth := aValue;
    Realign;
    Invalidate;
{}END {SetBevelWidth};




{}PROCEDURE   TKustomGroupBox.SetBorderStyle(aValue:TGroupBorderStyle);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    IF FBorderStyle <> aValue THEN BEGIN
      FBorderStyle := aValue;
      RecreateWnd;
    END {IF};
{}END {SetBorderStyle};




{}PROCEDURE   TKustomGroupBox.SetBorderWidth(aValue:TGroupBorderWidth);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    FBorderWidth := aValue;
    Realign;
    Invalidate;
{}END {SetBorderWidth};




{}PROCEDURE   TKustomGroupBox.SetColor(aIndex:INTEGER;aValue:TColor);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    CASE aIndex OF
      xBoxColor    : BEGIN
        IF FiBoxColor <> aValue THEN BEGIN
          FiBoxColor := aValue;
          FfParentColor := FALSE;
          Invalidate;
        END {IF};
      END;

      xTxtColor    : BEGIN
        IF FiTxtColor <> aValue THEN BEGIN
          FiTxtColor := aValue;
          FfParentColor := FALSE;
          Invalidate;
        END {IF};
      END;
    END {CASE};
{}END {SetColor};




{}PROCEDURE   TKustomGroupBox.SetkParentColor(aValue:BOOLEAN);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    IF FfParentColor <> aValue THEN BEGIN
      FfParentColor := aValue;
      IF Parent <> NIL THEN BEGIN
        Perform(CM_PARENTCOLORCHANGED, 0, 0);
      END {IF};
    END {IF};
{}END {SetKParentColor};




{}PROCEDURE   TKustomGroupBox.CMParentColorChanged(VAR aMessage: TMessage);
{+M
---------------------------------------------------------------------------
  Remarks     -
---------------------------------------------------------------------------}
  BEGIN
    IF FfParentColor THEN BEGIN
      IF aMessage.wParam <> 0 THEN BEGIN
        BoxColor := TColor(aMessage.lParam);
      END ELSE BEGIN
        BoxColor := TKontrol(Parent).Color;
      END {IF};

      TextColor:= BoxColor;
      FfParentColor := TRUE;
    END {IF};
{}END {CMParentColorChanged};




END {KCtrls}.
